// Copyright (C) 2016  University of Malta

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

extern crate clap;

mod error;
mod input;
mod number;
mod output;
mod package;
mod parse;
mod pass;
mod tree;

use std::io;
use std::process;

use std::error::Error;
use std::io::Write;

#[derive(Copy, Clone, PartialEq, Debug)]
pub struct Overwrite(bool);

fn translate(filenames: clap::Values, overwrite: Overwrite) -> Result<(), error::Error> {
    let mut package = package::Package::new();
    for filename in filenames {
        package.read_source(filename)?;
    }
    package.parse()?;
    println!("{:?}", package);
    package.process()?;
    println!("{:?}", package);
    package.output(overwrite)?;
    Ok(())
}

fn main() {
    let app = clap::App::new("rox")
        .version("0.1.1")
        .author("Trevor Spiteri <trevor.spiteri@um.edu.mt>")
        .about("Translates rox files into vhdl.")
        .arg(clap::Arg::with_name("INPUT")
            .help("Sets the input files to translate")
            .required(true)
            .multiple(true)
            .index(1))
        .arg(clap::Arg::with_name("overwrite")
            .short("o")
            .long("overwrite")
            .help("Overwrites existing target files"));
    let matches = app.get_matches();
    let filenames = matches.values_of("INPUT").unwrap();
    let overwrite = Overwrite(matches.is_present("overwrite"));
    match translate(filenames, overwrite) {
        Ok(_) => {}
        Err(e) => {
            writeln!(&mut io::stderr(), "{}", e.description()).unwrap();
            process::exit(2);
        }
    }
}
