// Copyright (C) 2016  University of Malta

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#![allow(dead_code)]

mod float;
mod gmp;
mod integer;
mod mpfr;
mod rational;

pub use self::integer::Integer;
pub use self::rational::Rational;
pub use self::float::{Float, Exp, Prec};
pub use self::float::{Round, Constant, Special};
pub use self::float::get_exp_range;

pub trait Set<T> {
    fn set(&mut self, t: T);
}

pub trait Get<T> {
    fn get(&self) -> T;
}

pub trait FromPrec<T> {
    fn from_prec(Prec, T) -> Self;
}

pub trait FromPrecRound<T> {
    fn from_prec_round(Prec, T, Round) -> Self;
}

pub trait SetRound<T> {
    fn set_round(&mut self, T, Round);
}

pub trait GetRound<T> {
    fn get_round(&self, Round) -> T;
}

pub trait AddRound<T> {
    type Output;
    fn add_round(self, T, Round) -> Self::Output;
}

pub trait SubRound<T> {
    type Output;
    fn sub_round(self, T, Round) -> Self::Output;
}

pub trait MulRound<T> {
    type Output;
    fn mul_round(self, T, Round) -> Self::Output;
}

pub trait DivRound<T> {
    type Output;
    fn div_round(self, T, Round) -> Self::Output;
}

pub trait ShlRound<T> {
    type Output;
    fn shl_round(self, T, Round) -> Self::Output;
}

pub trait ShrRound<T> {
    type Output;
    fn shr_round(self, T, Round) -> Self::Output;
}

pub trait Pow<T> {
    type Output;
    fn pow(self, T) -> Self::Output;
}

pub trait PowAssign<T> {
    fn pow_assign(&mut self, T);
}

pub trait PowRound<T> {
    type Output;
    fn pow_round(self, T, Round) -> Self::Output;
}
